using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;


public class AttachedPositionFunctionBehaviour : FactWrapper
{
    private Transform startcondition;

    public List<FunctionCallFact> f_t_to_pos;
    //public List<FunctionFact<float, Quaternion>> f_t_to_rot;
    //public List<FunctionFact<float, Vector3>> f_t_to_scale;

    private int active_func_ind = -1;
    private float time = 0;

    private bool _done;
    public bool done
    {
        get => _done;
        private set
        {
            if (value)
            {
                if(!repeat)
                    Destroy(gameObject);

                Reset();
                StartExecution();
            }
            else
                _done = value; // false
        }
    }

    public bool repeat = false;

    public bool executing
    {
        get { return active_func_ind >= 0 && active_func_ind < f_t_to_pos.Count; }
        set { if (value) StartExecution(); else StoppExecution(); }
    }

    public bool original = true;

    public void NewExecutingInstance()
    {
        if (!original)
            return;

        AttachedPositionFunctionBehaviour cloneComp =
            Instantiate(gameObject)
            .GetComponent<AttachedPositionFunctionBehaviour>();

        cloneComp.original = false;
        cloneComp.FactUpdated();
        cloneComp.StartExecution();
    }

    public override void HighlightAs(FactMaterials tmpState)
        => gameObject.GetComponent<FactObject>().HighlightAs(tmpState);

    protected override void FactUpdated(){
        startcondition = transform;
        f_t_to_pos = (Fact as AttachedPositionFunction).FunctionCallFacts.ToList();
    }

    protected override void _OnEnable()
    {
        CommunicationEvents.StartT0Event.AddListener(NewExecutingInstance);
    }

    protected override void _OnDisable()
    {
        CommunicationEvents.StartT0Event.RemoveListener(NewExecutingInstance);
    }

    void Update()
    {
        if (!executing)
            return;

        while (time > f_t_to_pos[active_func_ind].Domain.t_n)
        {
            active_func_ind++;

            if (!executing)
            {
                done = true;
                return;
            }

            time = time
                - f_t_to_pos[active_func_ind - 1].Domain.t_n
                + f_t_to_pos[active_func_ind + 0].Domain.t_0;
        }

        transform.position = (Vector3)f_t_to_pos[active_func_ind].Call(time)[0];
        time += Time.deltaTime;
    }

    public void Reset()
    {
        active_func_ind = -1;
        time = f_t_to_pos.FirstOrDefault().Domain.t_0;
        transform.position = startcondition.position;
        done = false;
    }

    public void StartExecution()
    {
        Reset();
        active_func_ind = 0;
    }

    public void StoppExecution()
    {
        active_func_ind = -1;
    }
}
