using UnityEngine;

/// <summary>
/// a Gadget that checks whether two given circles have equal size and if yes it returns an EqualCirclesFact
/// </summary>
public class EqualCircleGadget : Gadget
{
    /// \copydoc Gadget.s_type
    [Newtonsoft.Json.JsonProperty]
    protected static new string s_type = "EqualCircles";


    protected override void _Hit(RaycastHit[] hit)
    {
        string tempFactId = hit[0].transform.GetComponent<FactObject>().URI;
        if (!Workflow.Contains(tempFactId)) // filters double hits / exact ones
            Workflow.Add(tempFactId);

        switch (Workflow.Count)
        {
            case 1: //If no circle was selected before
                break;

            case 2: //If the first circle got already selected

                CircleFact tempFact = (CircleFact)FactOrganizer.AllFacts[Workflow[0]];
                CircleFact FirstCircle = (CircleFact)FactOrganizer.AllFacts[tempFactId];

                if (Mathf.Approximately(FirstCircle.radius, tempFact.radius))
                    FactManager.AddEqualCirclesFact  (Workflow[0], tempFactId);

                else
                    FactManager.AddUnEqualCirclesFact(Workflow[0], tempFactId);

                ResetGadget();
                break;
        }
    }
}
