﻿using UnityEngine;

public class PoleTool : Gadget
//Acts as a Pendulum starting at a Point
{
    /// \copydoc Gadget.s_type
    [Newtonsoft.Json.JsonProperty]
    protected static new string s_type = "PoleTool";

    public float poleHeight = 1f;


    protected override void _Enable()
    {
        ActivateLineDrawing();
    }

    protected override void _Hit(RaycastHit[] hit)
    {
        if (!Physics.Raycast(
                GadgetBehaviour.Cursor.transform.position + Vector3.up * (float)Math3d.vectorPrecission
                , Vector3.down, MaxHeight + (float)Math3d.vectorPrecission
                , this.SecondaryLayerMask))
            return;

        UpdateLineDrawing();

        if (hit[0].transform.gameObject.layer == LayerMask.NameToLayer("Point"))
        {
            Workflow.Add(hit[0].transform.gameObject.GetComponent<FactObject>().URI);

            string pid2 = FactManager.AddPointFact(GetPosition(1), Vector3.up, gadget: this).Id;
            FactManager.AddLineFact(Workflow[0], pid2, true, gadget: this);
        }
        else
        {
            FactManager.AddPointFact(hit[0], gadget: this);
        }

        ResetGadget();
        ActivateLineDrawing();
    }

    protected override void _ActivateLineDrawing()
    {
        GadgetBehaviour.LineRenderer.positionCount = 2;

        GadgetBehaviour.LineRenderer.startWidth = 0.095f;
        GadgetBehaviour.LineRenderer.endWidth = 0.095f;
    }

    //Updates the points of the Lines when baseLine was selected in LineMode
    protected override void _UpdateLineDrawing()
    {
        SetPosition(0, GadgetBehaviour.Cursor.transform.position);

        //Raycast upwoard
        if (Physics.Raycast(GetPosition(0), Vector3.up, out RaycastHit ceiling, poleHeight, this.SecondaryLayerMask))
            SetPosition(1, ceiling.point);
        else
            SetPosition(1, GetPosition(0) + Vector3.up * poleHeight);
    }
}
